#!/bin/bash
# A program which manages Ubuntu-it's web test server
# Copyright (C) 2015  Pietro Albini <pietroalbini@ubuntu.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published
# by the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; witout even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import os
import sys
import json
import subprocess

import click

from . import app
from . import utils


def error(message, *format):
    """Show an error message"""
    click.echo("managetests: error: {}".format(message.format(*format)))
    sys.exit(1)


@click.group()
@click.option("-r", "--root", help="The managetests root")
@click.pass_context
def cli(ctx, root):
    """Manage a managetests instance"""
    ctx.obj = {}
    skip_check = ["init"]

    if root is not None and "MANAGETESTS_ROOT" in os.environ:
        root = os.environ["MANAGETESTS_ROOT"]

    skip = ctx.invoked_subcommand in skip_check
    if root is None or not (skip or utils.is_root_valid(root)):
        error("please provide a valid root path")
    else:
        ctx.obj["root"] = os.path.abspath(root)


@cli.command("init")
@click.argument("git-url")
@click.pass_context
def init_command(ctx, git_url):
    """Init a new root directory"""
    root = ctx.obj["root"]

    # Check if the directory is empty
    if os.path.exists(root) and os.listdir(root):
        error("root directory not empty")

    os.makedirs(root, exist_ok=True)
    for dir in ("git", "envs", "caches", "public", "socks", "socks/branches"):
        os.mkdir(os.path.join(root, dir))

    with open(os.path.join(root, "config.json"), "w") as f:
        content = {
            "gitlab-url": "http://code.ubuntu-it.org",
            "gitlab-project": "ubuntu-it-web/www",
            "gitlab-token": "abcdefghi",
            "keep-branches": [
                "master",
            ],
        }
        json.dump(content, f, indent=4)
        f.write("\n")

    with open(os.path.join(root, "details.json"), "w") as f:
        content = {
            "branches": {},
        }
        json.dump(content, f, indent=4)
        f.write("\n")

    subprocess.call(["git", "clone", git_url, os.path.join(root, "git")])


@cli.command("run")
@click.pass_obj
def run_command(obj):
    """Run managetests"""
    root = obj["root"]

    with open(os.path.join(root, "config.json")) as f:
        config = json.load(f)

    inst = app.TestsManager(root, config)
    inst.run()
